@@ -28,13 +28,12 @@
 #include <unistd.h>
 #include <string.h>
 #include <time.h>
+#include <errno.h>
 #include <gtk/gtk.h>
 #include <gio/gio.h>
 
 #include <glib/gi18n.h>
 
-#include <libgnome/gnome-program.h>
-#include <libgnomeui/gnome-ui-init.h>
 #include <libnautilus-private/nautilus-module.h>
 #include <libnautilus-private/nautilus-icon-info.h>
 
@@ -191,9 +190,9 @@ present_autorun_for_software_dialog (GMount *mount)
 						     0,
 						     GTK_MESSAGE_OTHER,
 						     GTK_BUTTONS_CANCEL,
-						     _("<big><b>This media contains software intended to be automatically started. Would you like to run it?</b></big>"));
+						     _("<big><b>This medium contains software intended to be automatically started. Would you like to run it?</b></big>"));
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-						  _("The software will run directly from the media \"%s\". "
+						  _("The software will run directly from the medium \"%s\". "
 						    "You should never run software that you don't trust.\n"
 						    "\n"
 						    "If in doubt, press Cancel."),
@@ -253,9 +252,7 @@ main (int argc, char *argv[])
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
-	gnome_program_init ("nautilus-autorun-software", VERSION,
-			    LIBGNOMEUI_MODULE, argc, argv,
-			    NULL, NULL);
+	gtk_init (&argc, &argv);
 
         if (argc != 2) {
                 goto out;
