@@ -39,15 +39,14 @@
 #include "nautilus-window-private.h"
 #include "nautilus-desktop-window.h"
 #include "nautilus-search-bar.h"
-#include <gtk/gtkmain.h>
-#include <gtk/gtkaboutdialog.h>
-#include <gtk/gtkenums.h>
-#include <gtk/gtkversion.h>
+#include <gtk/gtk.h>
 #include <gio/gio.h>
 #include <glib/gi18n.h>
-#include <libgnomeui/gnome-help.h>
+#include <eel/eel-gtk-extensions.h>
+#include <eel/eel-preferences.h>
 #include <libnautilus-extension/nautilus-menu-provider.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
+#include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-icon-names.h>
 #include <libnautilus-private/nautilus-ui-utilities.h>
 #include <libnautilus-private/nautilus-module.h>
@@ -111,9 +110,26 @@ bookmark_holder_free_cover (gpointer callback_data, GClosure *closure)
 	bookmark_holder_free (callback_data);
 }
 
+static gboolean
+should_open_in_new_tab (void)
+{
+	/* FIXME this is duplicated */
+	GdkEvent *event;
+
+	event = gtk_get_current_event ();
+	if (event->type == GDK_BUTTON_PRESS || event->type == GDK_BUTTON_RELEASE) {
+		return event->button.button == 2;
+	}
+
+	gdk_event_free (event);
+
+	return FALSE;
+}
+
 static void
 activate_bookmark_in_menu_item (GtkAction *action, gpointer user_data)
 {
+	NautilusWindowSlot *slot;
         BookmarkHolder *holder;
         GFile *location;
 
@@ -123,7 +139,10 @@ activate_bookmark_in_menu_item (GtkAction *action, gpointer user_data)
 		holder->failed_callback (holder->window, holder->bookmark);
 	} else {
 	        location = nautilus_bookmark_get_location (holder->bookmark);
-	        nautilus_window_go_to (holder->window, location);
+		slot = nautilus_window_get_active_slot (holder->window);
+	        nautilus_window_slot_go_to (slot, 
+					    location, 
+					    should_open_in_new_tab ());
 	        g_object_unref (location);
         }
 }
@@ -188,10 +207,16 @@ nautilus_menus_append_bookmark_to_menu (NautilusWindow *window,
 }
 
 static void
-action_close_window_callback (GtkAction *action, 
-			      gpointer user_data)
+action_close_window_slot_callback (GtkAction *action,
+				   gpointer user_data)
 {
-	nautilus_window_close (NAUTILUS_WINDOW (user_data));
+	NautilusWindow *window;
+	NautilusWindowSlot *slot;
+
+	window = NAUTILUS_WINDOW (user_data);
+	slot = nautilus_window_get_active_slot (window);
+
+	nautilus_window_slot_close (slot);
 }
 
 static void
@@ -199,9 +224,12 @@ action_connect_to_server_callback (GtkAction *action,
 				   gpointer user_data)
 {
 	NautilusWindow *window = NAUTILUS_WINDOW (user_data);
+	NautilusWindowSlot *slot;
 	GtkWidget *dialog;
 	GFile *location;
-	location = nautilus_window_get_location (window);
+
+	slot = nautilus_window_get_active_slot (window);
+	location = nautilus_window_slot_get_location (slot);
 	dialog = nautilus_connect_server_dialog_new (window, location);
 	if (location) {
 		g_object_unref (location);
@@ -210,36 +238,17 @@ action_connect_to_server_callback (GtkAction *action,
 	gtk_widget_show (dialog);
 }
 
-static gboolean
-have_burn_uri (void)
-{
-	static gboolean initialized = FALSE;
-	static gboolean res;
-	GVfs *vfs;
-	int i;
-	const gchar * const * supported_uri_schemes;
-
-	if (!initialized) {
-		vfs = g_vfs_get_default ();
-		supported_uri_schemes = g_vfs_get_supported_uri_schemes (vfs);
-
-		res = FALSE;
-		for (i = 0; supported_uri_schemes != NULL && supported_uri_schemes[i] != NULL; i++) {
-			if (strcmp ("burn", supported_uri_schemes[i]) == 0) {
-				res = TRUE;
-				break;
-			}
-		}
-		initialized = TRUE;
-	}
-	return res;
-}
-
 static void
 action_stop_callback (GtkAction *action, 
 		      gpointer user_data)
 {
-	nautilus_window_stop_loading (NAUTILUS_WINDOW (user_data));
+	NautilusWindow *window;
+	NautilusWindowSlot *slot;
+
+	window = NAUTILUS_WINDOW (user_data);
+	slot = nautilus_window_get_active_slot (window);
+
+	nautilus_window_slot_stop_loading (slot);
 }
 
 static void
@@ -254,17 +263,31 @@ static void
 action_home_callback (GtkAction *action, 
 		      gpointer user_data) 
 {
-	nautilus_window_go_home (NAUTILUS_WINDOW (user_data));
+	NautilusWindow *window;
+	NautilusWindowSlot *slot;
+
+	window = NAUTILUS_WINDOW (user_data);
+	slot = nautilus_window_get_active_slot (window);
+
+	nautilus_window_slot_go_home (slot, 
+				      should_open_in_new_tab ());
 }
 
 static void
 action_go_to_computer_callback (GtkAction *action, 
 				gpointer user_data) 
 {
+	NautilusWindow *window;
+	NautilusWindowSlot *slot;
 	GFile *computer;
+
+	window = NAUTILUS_WINDOW (user_data);
+	slot = nautilus_window_get_active_slot (window);
+
 	computer = g_file_new_for_uri (COMPUTER_URI);
-	nautilus_window_go_to (NAUTILUS_WINDOW (user_data),
-			       computer);
+	nautilus_window_slot_go_to (slot,
+				    computer,
+				    should_open_in_new_tab ());
 	g_object_unref (computer);
 }
 
@@ -272,10 +295,17 @@ static void
 action_go_to_network_callback (GtkAction *action, 
 				gpointer user_data) 
 {
+	NautilusWindow *window;
+	NautilusWindowSlot *slot;
 	GFile *network;
+
+	window = NAUTILUS_WINDOW (user_data);
+	slot = nautilus_window_get_active_slot (window);
+
 	network = g_file_new_for_uri (NETWORK_URI);
-	nautilus_window_go_to (NAUTILUS_WINDOW (user_data),
-			       network);
+	nautilus_window_slot_go_to (slot,
+				    network,
+				    should_open_in_new_tab ());
 	g_object_unref (network);
 }
 
@@ -283,14 +313,20 @@ static void
 action_go_to_templates_callback (GtkAction *action,
 				 gpointer user_data) 
 {
+	NautilusWindow *window;
+	NautilusWindowSlot *slot;
 	char *path;
 	GFile *location;
 
+	window = NAUTILUS_WINDOW (user_data);
+	slot = nautilus_window_get_active_slot (window);
+
 	path = nautilus_get_templates_directory ();
 	location = g_file_new_for_path (path);
 	g_free (path);
-	nautilus_window_go_to (NAUTILUS_WINDOW (user_data),
-			       location);
+	nautilus_window_slot_go_to (slot,
+				    location,
+				    should_open_in_new_tab ());
 	g_object_unref (location);
 }
 
@@ -298,23 +334,18 @@ static void
 action_go_to_trash_callback (GtkAction *action, 
 			     gpointer user_data) 
 {
+	NautilusWindow *window;
+	NautilusWindowSlot *slot;
 	GFile *trash;
-	trash = g_file_new_for_uri ("trash:///");
-	nautilus_window_go_to (NAUTILUS_WINDOW (user_data),
-			       trash);
-	g_object_unref (trash);
-}
 
-static void
-action_go_to_burn_cd_callback (GtkAction *action,
-			       gpointer user_data) 
-{
-	GFile *burn;
-	burn = g_file_new_for_uri (BURN_CD_URI);
-	nautilus_window_go_to (NAUTILUS_WINDOW (user_data),
-			       burn);
-	g_object_unref (burn);
-	
+	window = NAUTILUS_WINDOW (user_data);
+	slot = nautilus_window_get_active_slot (window);
+
+	trash = g_file_new_for_uri ("trash:///");
+	nautilus_window_slot_go_to (slot,
+				    trash,
+				    should_open_in_new_tab ());
+	g_object_unref (trash);
 }
 
 static void
@@ -345,6 +376,48 @@ action_zoom_normal_callback (GtkAction *action,
 	nautilus_window_zoom_to_default (NAUTILUS_WINDOW (user_data));
 }
 
+static void
+action_show_hidden_files_callback (GtkAction *action, 
+				   gpointer callback_data)
+{
+	NautilusWindow *window;
+	NautilusWindowShowHiddenFilesMode mode;
+
+	window = NAUTILUS_WINDOW (callback_data);
+
+	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action))) {
+		mode = NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_ENABLE;
+	} else {
+		mode = NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_DISABLE;
+	}
+
+	nautilus_window_info_set_hidden_files_mode (window, mode);
+}
+
+static void
+show_hidden_files_preference_callback (gpointer callback_data)
+{
+	NautilusWindow *window;
+	GtkAction *action;
+
+	window = NAUTILUS_WINDOW (callback_data);
+
+	if (window->details->show_hidden_files_mode == NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_DEFAULT) {
+		action = gtk_action_group_get_action (window->details->main_action_group, NAUTILUS_ACTION_SHOW_HIDDEN_FILES);
+		g_assert (GTK_IS_ACTION (action));
+
+		/* update button */
+		g_signal_handlers_block_by_func (action, action_show_hidden_files_callback, window);
+		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
+					      eel_preferences_get_boolean (NAUTILUS_PREFERENCES_SHOW_HIDDEN_FILES));
+		g_signal_handlers_unblock_by_func (action, action_show_hidden_files_callback, window);
+
+		/* inform views */
+		nautilus_window_info_set_hidden_files_mode (window, NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_DEFAULT);
+
+	}
+}
+
 static void
 preferences_respond_callback (GtkDialog *dialog,
 			      gint response_id)
@@ -442,10 +515,9 @@ action_about_nautilus_callback (GtkAction *action,
 	gtk_show_about_dialog (GTK_WINDOW (user_data),
 			       "program-name", _("Nautilus"),
 			       "version", VERSION,
-			       "comments", _("Nautilus is a graphical shell "
-					     "for GNOME that makes it "
-					     "easy to manage your files "
-					     "and the rest of your system."),
+			       "comments", _("Nautilus lets you organize "
+					     "files and folders, both on "
+					     "your computer and online."),
 			       "copyright", _("Copyright \xC2\xA9 1999-2008 "
 					      "The Nautilus authors"),
 			       "license", license_trans,
@@ -470,7 +542,7 @@ static void
 action_up_callback (GtkAction *action, 
 		     gpointer user_data) 
 {
-	nautilus_window_go_up (NAUTILUS_WINDOW (user_data), FALSE);
+	nautilus_window_go_up (NAUTILUS_WINDOW (user_data), FALSE, should_open_in_new_tab ());
 }
 
 static void
@@ -489,9 +561,9 @@ action_nautilus_manual_callback (GtkAction *action,
 			gtk_window_get_screen (GTK_WINDOW (window)),
 			"gnome-help", &error);
 	} else {
-		gnome_help_display_desktop_on_screen (
-			NULL, "user-guide", "user-guide.xml", "gosnautilus-1",
-			gtk_window_get_screen (GTK_WINDOW (window)), &error);
+		gtk_show_uri (gtk_window_get_screen (GTK_WINDOW (window)),
+			      "ghelp:user-guide#gosnautilus-1",
+			      gtk_get_current_event_time (), &error);
 	}
 
 	if (error) {
@@ -518,7 +590,7 @@ menu_item_select_cb (GtkMenuItem *proxy,
 	GtkAction *action;
 	char *message;
 
-	action = g_object_get_data (G_OBJECT (proxy),  "gtk-action");
+	action = gtk_widget_get_action (GTK_WIDGET (proxy));
 	g_return_if_fail (action != NULL);
 	
 	g_object_get (G_OBJECT (action), "tooltip", &message, NULL);
@@ -537,18 +609,83 @@ menu_item_deselect_cb (GtkMenuItem *proxy,
 			   window->details->help_message_cid);
 }
 
+static GtkWidget *
+get_event_widget (GtkWidget *proxy)
+{
+	GtkWidget *widget;
+
+	/**
+	 * Finding the interesting widget requires internal knowledge of
+	 * the widgets in question. This can't be helped, but by keeping
+	 * the sneaky code in one place, it can easily be updated.
+	 */
+	if (GTK_IS_MENU_ITEM (proxy)) {
+		/* Menu items already forward middle clicks */
+		widget = NULL;
+	} else if (GTK_IS_MENU_TOOL_BUTTON (proxy)) {
+		widget = eel_gtk_menu_tool_button_get_button (GTK_MENU_TOOL_BUTTON (proxy));
+	} else if (GTK_IS_TOOL_BUTTON (proxy)) {
+		/* The tool button's button is the direct child */
+		widget = gtk_bin_get_child (GTK_BIN (proxy));
+	} else if (GTK_IS_BUTTON (proxy)) {
+		widget = proxy;
+	} else {
+		/* Don't touch anything we don't know about */
+		widget = NULL;
+	}
+
+	return widget;
+}
+
+static gboolean
+proxy_button_press_event_cb (GtkButton *button,
+			     GdkEventButton *event,
+			     gpointer user_data)
+{
+	if (event->button == 2) {
+		gtk_button_pressed (button);
+	}
+
+	return FALSE;
+}
+
+static gboolean
+proxy_button_release_event_cb (GtkButton *button,
+			       GdkEventButton *event,
+			       gpointer user_data)
+{
+	if (event->button == 2) {
+		gtk_button_released (button);
+	}
+
+	return FALSE;
+}
+
 static void
 disconnect_proxy_cb (GtkUIManager *manager,
 		     GtkAction *action,
 		     GtkWidget *proxy,
 		     NautilusWindow *window)
 {
+	GtkWidget *widget;
+
 	if (GTK_IS_MENU_ITEM (proxy)) {
 		g_signal_handlers_disconnect_by_func
 			(proxy, G_CALLBACK (menu_item_select_cb), window);
 		g_signal_handlers_disconnect_by_func
 			(proxy, G_CALLBACK (menu_item_deselect_cb), window);
 	}
+
+	widget = get_event_widget (proxy);
+	if (widget) {
+		g_signal_handlers_disconnect_by_func (widget,
+						      G_CALLBACK (proxy_button_press_event_cb),
+						      action);
+		g_signal_handlers_disconnect_by_func (widget,
+						      G_CALLBACK (proxy_button_release_event_cb),
+						      action);
+	}
+
 }
 
 static void
@@ -584,6 +721,15 @@ connect_proxy_cb (GtkUIManager *manager,
 		}
 	}
 	
+	widget = get_event_widget (proxy);
+	if (widget) {
+		g_signal_connect (widget, "button-press-event",
+				  G_CALLBACK (proxy_button_press_event_cb),
+				  action);
+		g_signal_connect (widget, "button-release-event",
+				  G_CALLBACK (proxy_button_release_event_cb),
+				  action);
+	}
 }
 
 static const GtkActionEntry main_entries[] = {
@@ -594,7 +740,7 @@ static const GtkActionEntry main_entries[] = {
   /* name, stock id */         { "Close", GTK_STOCK_CLOSE,
   /* label, accelerator */       N_("_Close"), "<control>W",
   /* tooltip */                  N_("Close this folder"),
-                                 G_CALLBACK (action_close_window_callback) },
+                                 G_CALLBACK (action_close_window_slot_callback) },
                                { "Backgrounds and Emblems", NULL,
                                  N_("_Backgrounds and Emblems..."),               
                                  NULL, N_("Display patterns, colors, and emblems that can be used to customize appearance"),
@@ -630,7 +776,7 @@ static const GtkActionEntry main_entries[] = {
                                  G_CALLBACK (action_about_nautilus_callback) },
   /* name, stock id */         { "Zoom In", GTK_STOCK_ZOOM_IN,
   /* label, accelerator */       N_("Zoom _In"), "<control>plus",
-  /* tooltip */                  N_("Show the contents in more detail"),
+  /* tooltip */                  N_("Increase the view size"),
                                  G_CALLBACK (action_zoom_in_callback) },
   /* name, stock id */         { "ZoomInAccel", NULL,
   /* label, accelerator */       "ZoomInAccel", "<control>equal",
@@ -642,7 +788,7 @@ static const GtkActionEntry main_entries[] = {
                                  G_CALLBACK (action_zoom_in_callback) },
   /* name, stock id */         { "Zoom Out", GTK_STOCK_ZOOM_OUT,
   /* label, accelerator */       N_("Zoom _Out"), "<control>minus",
-  /* tooltip */                  N_("Show the contents in less detail"),
+  /* tooltip */                  N_("Decrease the view size"),
                                  G_CALLBACK (action_zoom_out_callback) },
   /* name, stock id */         { "ZoomOutAccel", NULL,
   /* label, accelerator */       "ZoomOutAccel", "<control>KP_Subtract",
@@ -650,7 +796,7 @@ static const GtkActionEntry main_entries[] = {
                                  G_CALLBACK (action_zoom_out_callback) },
   /* name, stock id */         { "Zoom Normal", GTK_STOCK_ZOOM_100,
   /* label, accelerator */       N_("Normal Si_ze"), "<control>0",
-  /* tooltip */                  N_("Show the contents at the normal size"),
+  /* tooltip */                  N_("Use the normal view size"),
                                  G_CALLBACK (action_zoom_normal_callback) },
   /* name, stock id */         { "Connect to Server", NULL, 
   /* label, accelerator */       N_("Connect to _Server..."), NULL,
@@ -676,10 +822,14 @@ static const GtkActionEntry main_entries[] = {
   /* label, accelerator */       N_("_Trash"), NULL,
   /* tooltip */                  N_("Open your personal trash folder"),
                                  G_CALLBACK (action_go_to_trash_callback) },
-  /* name, stock id */         { "Go to Burn CD", NAUTILUS_ICON_BURN,
-  /* label, accelerator */       N_("CD/_DVD Creator"), NULL,
-  /* tooltip */                  N_("Open a folder into which you can drag files to burn to a CD or DVD"),
-                                 G_CALLBACK (action_go_to_burn_cd_callback) },
+};
+
+static const GtkToggleActionEntry main_toggle_entries[] = {
+  /* name, stock id */         { "Show Hidden Files", NULL,
+  /* label, accelerator */       N_("Show _Hidden Files"), "<control>H",
+  /* tooltip */                  N_("Toggle the display of hidden files in the current window"),
+                                 G_CALLBACK (action_show_hidden_files_callback),
+                                 TRUE },
 };
 
 /**
@@ -702,6 +852,9 @@ nautilus_window_initialize_menus (NautilusWindow *window)
 	gtk_action_group_add_actions (action_group, 
 				      main_entries, G_N_ELEMENTS (main_entries),
 				      window);
+	gtk_action_group_add_toggle_actions (action_group, 
+					     main_toggle_entries, G_N_ELEMENTS (main_toggle_entries),
+					     window);
 
 	action = gtk_action_group_get_action (action_group, NAUTILUS_ACTION_UP);
 	g_object_set (action, "short_label", _("_Up"), NULL);
@@ -709,6 +862,17 @@ nautilus_window_initialize_menus (NautilusWindow *window)
 	action = gtk_action_group_get_action (action_group, NAUTILUS_ACTION_HOME);
 	g_object_set (action, "short_label", _("_Home"), NULL);
 
+	action = gtk_action_group_get_action (action_group, NAUTILUS_ACTION_SHOW_HIDDEN_FILES);
+	g_signal_handlers_block_by_func (action, action_show_hidden_files_callback, window);
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
+				      eel_preferences_get_boolean (NAUTILUS_PREFERENCES_SHOW_HIDDEN_FILES));
+	g_signal_handlers_unblock_by_func (action, action_show_hidden_files_callback, window);
+
+
+	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_SHOW_HIDDEN_FILES,
+						  show_hidden_files_preference_callback,
+						  window, G_OBJECT (window));
+
 	window->details->ui_manager = gtk_ui_manager_new ();
 	ui_manager = window->details->ui_manager;
 	gtk_window_add_accel_group (GTK_WINDOW (window),
@@ -728,25 +892,10 @@ nautilus_window_initialize_menus (NautilusWindow *window)
 	nautilus_window_initialize_bookmarks_menu (window);
 }
 
-void
-nautilus_window_initialize_menus_constructed (NautilusWindow *window)
-{
-	GtkAction *action;
-
-	/* Don't call have_burn_uri() for the desktop window, as this is a very
-	 * expensive operation during login (around 1 second) ---
-	 * have_burn_uri() has to create a "burn:///" URI, which causes
-	 * gnome-vfs to link in libmapping.so from nautilus-cd-burner.
-	 */
-	if (nautilus_window_has_menubar_and_statusbar (window) && !have_burn_uri ()) {
-		action = gtk_action_group_get_action (window->details->main_action_group, NAUTILUS_ACTION_GO_TO_BURN_CD);
- 		gtk_action_set_visible (action, FALSE);
- 	}
-}
-
 static GList *
 get_extension_menus (NautilusWindow *window)
 {
+	NautilusWindowSlot *slot;
 	GList *providers;
 	GList *items;
 	GList *l;
@@ -754,6 +903,8 @@ get_extension_menus (NautilusWindow *window)
 	providers = nautilus_module_get_extensions_for_type (NAUTILUS_TYPE_MENU_PROVIDER);
 	items = NULL;
 
+	slot = nautilus_window_get_active_slot (window);
+
 	for (l = providers; l != NULL; l = l->next) {
 		NautilusMenuProvider *provider;
 		GList *file_items;
@@ -761,7 +912,7 @@ get_extension_menus (NautilusWindow *window)
 		provider = NAUTILUS_MENU_PROVIDER (l->data);
 		file_items = nautilus_menu_provider_get_background_items (provider,
 									  GTK_WIDGET (window),
-									  window->details->viewed_file);
+									  slot->viewed_file);
 		items = g_list_concat (items, file_items);
 	}
 
@@ -770,15 +921,76 @@ get_extension_menus (NautilusWindow *window)
 	return items;
 }
 
+static void
+add_extension_menu_items (NautilusWindow *window,
+			  guint merge_id,
+			  GtkActionGroup *action_group,
+			  GList *menu_items,
+			  const char *subdirectory)
+{
+	GtkUIManager *ui_manager;
+	GList *l;
+
+	ui_manager = window->details->ui_manager;
+	
+	for (l = menu_items; l; l = l->next) {
+		NautilusMenuItem *item;
+		NautilusMenu *menu;
+		GtkAction *action;
+		char *path;
+		
+		item = NAUTILUS_MENU_ITEM (l->data);
+		
+		g_object_get (item, "menu", &menu, NULL);
+		
+		action = nautilus_action_from_menu_item (item);
+		gtk_action_group_add_action (action_group, action);
+		
+		path = g_build_path ("/", POPUP_PATH_EXTENSION_ACTIONS, subdirectory, NULL);
+		gtk_ui_manager_add_ui (ui_manager,
+				       merge_id,
+				       path,
+				       gtk_action_get_name (action),
+				       gtk_action_get_name (action),
+				       (menu != NULL) ? GTK_UI_MANAGER_MENU : GTK_UI_MANAGER_MENUITEM,
+				       FALSE);
+		g_free (path);
+
+		path = g_build_path ("/", MENU_PATH_EXTENSION_ACTIONS, subdirectory, NULL);
+		gtk_ui_manager_add_ui (ui_manager,
+				       merge_id,
+				       path,
+				       gtk_action_get_name (action),
+				       gtk_action_get_name (action),
+				       (menu != NULL) ? GTK_UI_MANAGER_MENU : GTK_UI_MANAGER_MENUITEM,
+				       FALSE);
+		g_free (path);
+
+		/* recursively fill the menu */		       
+		if (menu != NULL) {
+			char *subdir;
+			GList *children;
+			
+			children = nautilus_menu_get_items (menu);
+			
+			subdir = g_build_path ("/", subdirectory, "/", gtk_action_get_name (action), NULL);
+			add_extension_menu_items (window,
+						  merge_id,
+						  action_group,
+						  children,
+						  subdir);
+
+			nautilus_menu_item_list_free (children);
+			g_free (subdir);
+		}			
+	}
+}
+
 void
 nautilus_window_load_extension_menus (NautilusWindow *window)
 {
-	NautilusMenuItem *item;
 	GtkActionGroup *action_group;
-	GtkAction *action;
 	GList *items;
-	GList *l;
-	int i;
 	guint merge_id;
 
 	if (window->details->extensions_menu_merge_id != 0) {
@@ -803,35 +1015,12 @@ nautilus_window_load_extension_menus (NautilusWindow *window)
 
 	items = get_extension_menus (window);
 
-	for (l = items, i = 0; l != NULL; l = l->next, i++) {
-		item = NAUTILUS_MENU_ITEM (l->data);
+	if (items != NULL) {
+		add_extension_menu_items (window, merge_id, action_group, items, "");
 
-		action = nautilus_action_from_menu_item (item);
-		gtk_action_group_add_action (action_group,
-					     GTK_ACTION (action));
-		g_object_unref (action);
-		
-		gtk_ui_manager_add_ui (window->details->ui_manager,
-				       merge_id,
-				       MENU_PATH_EXTENSION_ACTIONS,
-				       gtk_action_get_name (action),
-				       gtk_action_get_name (action),
-				       GTK_UI_MANAGER_MENUITEM,
-				       FALSE);
-
-		gtk_ui_manager_add_ui (window->details->ui_manager,
-				       merge_id,
-				       POPUP_PATH_EXTENSION_ACTIONS,
-				       gtk_action_get_name (action),
-				       gtk_action_get_name (action),
-				       GTK_UI_MANAGER_MENUITEM,
-				       FALSE);
-
-		
-		g_object_unref (item);
+		g_list_foreach (items, (GFunc) g_object_unref, NULL);
+		g_list_free (items);
 	}
-
-	g_list_free (items);
 }
 
 
